@@ -2,9 +2,11 @@
 
 import sys
 import gettext
+import gi
+gi.require_version('Gtk', '3.0')
+gi.require_version('GnomeBluetooth', '1.0')
 from gi.repository import Gtk, Gdk, GnomeBluetooth, Gio
 import rfkillMagic
-import blueberrySettings
 import subprocess
 
 # i18n
@@ -17,11 +19,11 @@ class BluetoothTray:
             debug = True
 
         self.rfkill = rfkillMagic.Interface(self.update_icon_callback, debug)
-        self.settings = blueberrySettings.Settings()
-        self.settings.gsettings.connect("changed::tray-enabled", self.on_settings_changed_cb)
+        self.settings = Gio.Settings("org.blueberry")
+        self.settings.connect("changed::tray-enabled", self.on_settings_changed_cb)
 
         # If we have no adapter, or disabled tray, end early
-        if (not self.rfkill.have_adapter) or (not self.settings.get_tray_enabled()):
+        if (not self.rfkill.have_adapter) or (not self.settings.get_boolean("tray-enabled")):
             self.rfkill.terminate()
             sys.exit(0)
 
@@ -39,7 +41,7 @@ class BluetoothTray:
         self.update_icon_callback(None, None, None)
 
     def on_settings_changed_cb(self, setting, key, data=None):
-        if not self.settings.get_tray_enabled():
+        if not self.settings.get_boolean("tray-enabled"):
             self.terminate()
 
     def update_icon_callback(self, path=None, iter=None, data=None):
@@ -49,24 +51,23 @@ class BluetoothTray:
 
         if self.rfkill.hard_block or self.rfkill.soft_block:
             self.icon.set_from_icon_name("blueberry-tray-disabled")
-            self.icon.set_tooltip_text(_("Bluetooth"))
+            self.icon.set_tooltip_text(_("Bluetooth is disabled"))
         else:
             self.icon.set_from_icon_name("blueberry-tray")
             self.update_connected_state()
 
     def update_connected_state(self):
-        n_devices = self.get_n_devices()
+        connected_devices = self.get_connected_devices()
 
-        if n_devices > 0:
+        if len(connected_devices) > 0:
             self.icon.set_from_icon_name("blueberry-tray-active")
-            self.icon.set_tooltip_text(gettext.ngettext("Bluetooth: %d device connected" % n_devices, 
-                                                        "Bluetooth: %d devices connected" % n_devices,
-                                                        n_devices))
+            self.icon.set_tooltip_text(_("Bluetooth: Connected to %s") % (", ".join(connected_devices)))
         else:
             self.icon.set_from_icon_name("blueberry-tray")
             self.icon.set_tooltip_text(_("Bluetooth"))
 
-    def get_n_devices(self):
+    def get_connected_devices(self):
+        connected_devices = []
         default_iter = None
 
         iter = self.model.get_iter_first()
@@ -78,20 +79,17 @@ class BluetoothTray:
                 break
             iter = self.model.iter_next(iter)
 
-        if default_iter == None:
-            return False
+        if default_iter != None:
+            iter = self.model.iter_children(default_iter)
+            while iter:
+                connected = self.model.get_value(iter, GnomeBluetooth.Column.CONNECTED)
+                if connected:
+                    name = self.model.get_value(iter, GnomeBluetooth.Column.NAME)
+                    connected_devices.append(name)
 
-        n_devices = 0
+                iter = self.model.iter_next(iter)
 
-        iter = self.model.iter_children(default_iter)
-        while iter:
-            connected = self.model.get_value(iter, GnomeBluetooth.Column.CONNECTED)
-            if connected:
-                n_devices += 1
-
-            iter = self.model.iter_next(iter)
-
-        return n_devices
+        return connected_devices
 
     def on_activate(self, icon, data=None):
         subprocess.Popen(["blueberry"])
